package org.honours.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.util.List;

import org.expeditee.actions.Misc;
import org.expeditee.gui.*;
import org.expeditee.items.Item;
import org.expeditee.items.Text;

import org.honours.Main;
import org.honours.collection.*;

/**
 * This class is used for displaying the
 * various different overlays/interfaces available.
 * @author Korii
 */
public class Interfaces {
	
	//various frame names for the different interface overlays.
	public static final String MAIN_INTERFACE_OVERLAY = "interface-overlays1";
	
	public static final String LAYERS_BACK_BTN_OVERLAY = "interface-overlays3";
	public static final String TOGGLE_TOOLS_OVERLAY = "interface-overlays4";

	public static final String BLANK_TOOLBAR_OVERLAY = "interface-overlays6";
	
	//overlay displaying toolbar with "View Notes" option DISABLED.
	public static final String NAVIGATION_OVERLAY_VIEW_NOTES_UNCLICKABLE = "interface-overlays2";
	
	//overlay displaying toolbar with "View Notes" option ENABLED.
	public static final String NAVIGATION_OVERLAY_VIEW_NOTES_CLICKABLE = "interface-overlays11";
	
	//public static final String DISPLAY_LAYERS_OVERLAY = "interface-overlays5";
	//public static final String BLANK_DISPLAY_LAYERS_OVERLAY = "interface-overlays7";
	
	//TODO: Overlay for hiding notes.
	public static final String BLANK_HIDE_NOTES_OVERLAY = "interface-overlays9";
	
	public static final String BLANK_TOGGLER_OVERLAY = "interface-overlays10";
	
	private static final String AO_TEXT3 = "@ao:3";
	private static final String AO_TEXT4 = "@ao:4";

	//used to determine positioning of the various @ao:3/4 text items.
	private static int _mainOverAoX = 1859;
	private static int _mainOverAoY = 130;
	
	private static int _toolbarAoX = 1859;
	private static int _toolbarAoY = 130;
	
	private static int _toggleAoX = 1859;
	private static int _toggleAoY = 94;
	
	private static int _notesAoX = 1859;
	private static int _notesAoY = 58;
	
	public static final String OVERLAY_TOOLBAR_DATA = "overlayToolbar";
	
	/**
	 * This method sets up the main overlay on the profile
	 * frame.
	 * @param frame - frame to add the main overlay too.
	 */
	public static void setUpMainOverlay(Frame frame){
		
		
		
		List<Text> list = frame.getBodyTextItems(true);
		
		boolean found = false;
		int i = 0;
		
		while(i < list.size() && !found){
			
			Text text = list.get(i);
			
			if(text.getText().equals(AO_TEXT3)){
				if(text.hasLink()){
					if(text.getLink().equals(MAIN_INTERFACE_OVERLAY)){
						found = true;
					}
					
				}
			}
			i++;
		}
		
		if(!found){
			Text t = frame.addText(_mainOverAoX,_mainOverAoY,AO_TEXT3,null);
			t.setLink(MAIN_INTERFACE_OVERLAY);
			
			frame.setSaved();
			frame.setChanged(true);
		}
		
		FrameKeyboardActions.Refresh();
	}
	
	
	/** Method to set up overlay locations relative to the screen size. **/
	public static void setUpOverlayLocations(CollectionItem cItem){
		
		Frame frame = cItem.getFrame();
		//Dimension dim = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
		
		//Find the location of the frame name text item and use that for the overlay links.
		Item nameItem = frame.getNameItem();
		
		int x = nameItem.getX();
		
		_mainOverAoX = x;
		_toolbarAoX = x;
		_toggleAoX = x;
		_notesAoX = x;
		
	}
	
	public static void resetOverlayPosition(Text t) {
		Dimension maxSize = FrameGraphics.getMaxFrameSize();
		if (maxSize != null) {
			// setMaxWidth(maxSize.width);
			t.setPosition(maxSize.width - t.getBoundsWidth(), t.getBoundsHeight());
		}
	}
	

	/**
	 * Set up overlay which displays toolbar for collection items.
	 * @param cItem - the collection item to add the toolbar to (or rather its represented frame).
	 */
	public static void setUpToolbarOverlay(CollectionItem cItem) {
		
		Frame ciFrame = cItem.getFrame();
		List<Text> list = ciFrame.getBodyTextItems(true);
		
		boolean found = false;
		int i = 0;
		
		while(i < list.size() && !found){
			
			Text t = list.get(i);
			
			if(t.getText().equals(AO_TEXT3)){
				
				if(t.hasLink()){
					if(t.getLink().equals(NAVIGATION_OVERLAY_VIEW_NOTES_UNCLICKABLE)){
						found = true;
					}
				}
			}
			
			i++;
		}
		
		if(!found){
			
			Text newText = ciFrame.addText(_toolbarAoX,_toolbarAoY,AO_TEXT3,null);
			newText.setData(OVERLAY_TOOLBAR_DATA);
			newText.setLink(NAVIGATION_OVERLAY_VIEW_NOTES_UNCLICKABLE);
		}
	
		
	}

	/**
	 * Method to set up overlay which displays a "toggle toolbar" option/button.
	 * @param cItem - the collection item to add the toggle toolbar option to.
	 */
	public static void setUpToggleToolbarOverlay(CollectionItem cItem) {
		Frame ciFrame = cItem.getFrame();
		List<Text> list = ciFrame.getBodyTextItems(true);
		
		boolean found = false;
		int i = 0;
		
		while(i < list.size() && !found){
			
			Text t = list.get(i);
			
			if(t.getText().equals(AO_TEXT3)){
				
				if(t.hasLink()){
					
					if(t.getLink().equals(TOGGLE_TOOLS_OVERLAY)){
						found = true;
					}
				}
			}
			i++;
		}
		
		if(!found){
			Text newText = ciFrame.addText(_toggleAoX,_toggleAoY,AO_TEXT3,null);
			newText.setData("overlayViewToolbar");
			newText.setLink(TOGGLE_TOOLS_OVERLAY);
		}
		
	}

	/**
	 * Method to set up an overlay for displaying notes for
	 * the passed collection item.
	 * @param cItem - collection item to add notes to.
	 */
	public static void setUpNotesOverlay(CollectionItem cItem) {
		
		Frame ciFrame = cItem.getFrame();
		List<Text> list = ciFrame.getBodyTextItems(true);
		
		boolean found = false;
		String notesFrameFoundName = null;
		int i = 0;
		
			
		while(i < list.size() && !found){
			
			Text t = list.get(i);
			
			if(t.getText().equals(AO_TEXT4)){
				
				if(t.hasLink()){
					
					if(t.getLink().startsWith("notes")){
						found = true;
						notesFrameFoundName = t.getLink();
					}
					
					
				}
			}
			
			i++;
		}
		
		if(!found){

			//check for an overlay frame in notes frameset associated with this collection item.
			NoteLayer getNoteLayer = cItem.getNoteLayer();
			Frame getNotesOverlayFrame = null;
			
			if(getNoteLayer == null){
				
				//Create a new notelayer and frame
				getNoteLayer = new NoteLayer();
				getNoteLayer.setAssociatedItem(cItem);
				
				//Create new frame.
				int frameNumber = FrameIO.getLastNumber("notesmain") + 1;
				HonoursFrameIO.generateFrame(frameNumber, "notesmain");
				getNotesOverlayFrame = FrameIO.LoadFrame("notesmain"+frameNumber);
				getNoteLayer.setFrame(FrameIO.LoadFrame(getNotesOverlayFrame.getName()));
				cItem.setNoteLayer(getNoteLayer);
				
				//Add collectionItem back into collection and add collection back into collections list
				Collection matchingCollection = Collection.findCollection(cItem.getFrame());
				int collectionIndex = Collection.getCollectionPosition(matchingCollection);
				int collectionItemPosition = CollectionItem.getPositionInCollection(cItem, matchingCollection);
				
				matchingCollection.getItems().remove(collectionItemPosition);
				matchingCollection.getItems().add(collectionItemPosition,cItem);
				
				Main._collections.remove(collectionIndex);
				Main._collections.add(collectionIndex, matchingCollection);
				
			}else{
				getNotesOverlayFrame = getNoteLayer.getFrame();
			}
			
			Text newText = ciFrame.addText(_notesAoX,_notesAoY,AO_TEXT4,null);
			newText.setData("overlayNotes");
			newText.setLink(getNotesOverlayFrame.getName());
			
		}else{
			
			//Obtain link to toolbar and change it.
			Item toolbarLinkItem = Misc.getItemContainingData(OVERLAY_TOOLBAR_DATA, ciFrame);
			String toolbarOverlayLink = toolbarLinkItem.getLink();
			
			if(toolbarOverlayLink.equals(NAVIGATION_OVERLAY_VIEW_NOTES_UNCLICKABLE)){
				toolbarLinkItem.setLink(NAVIGATION_OVERLAY_VIEW_NOTES_CLICKABLE);
			}
			else if(toolbarOverlayLink.equals(NAVIGATION_OVERLAY_VIEW_NOTES_CLICKABLE)){
				//Check for actual text items on the notes frame.
				Frame checkOverlayFrame = FrameIO.LoadFrame(notesFrameFoundName);
				
				if(checkOverlayFrame.getTextItems().size() == 0){
					toolbarLinkItem.setLink(NAVIGATION_OVERLAY_VIEW_NOTES_UNCLICKABLE);
					System.err.println("NO NOTES for collection item: " + ciFrame.getName());
				}else{
					System.err.println(ciFrame.getName() + " " + checkOverlayFrame.getTextItems().size() + " ********");
				}
				
			}
			
		}
	}
	
}
